#define BLYNK_TEMPLATE_ID           ""
#define BLYNK_TEMPLATE_NAME         ""
#define BLYNK_AUTH_TOKEN            ""

#define BLYNK_PRINT Serial

#define CLOCK_BUTTON  V0
#define ALARM_SYSTEM_LISTENING_BUTTON V1
#define DISPLAY_BOX V2
#define TIME_SLIDER V3
#define ALARM_SYSTEM_ACTIVE_BUTTON V4

#define LED_PIN 5
#define BUZZER_PIN 21
#define NUM_LEDS 12

#define SOUND_SENSOR_PIN 34
#define LIGHT_SENSOR_PIN 35


#define I2S_DOUT      25
#define I2S_BCLK      27
#define I2S_LRC       26

#include <HTTPClient.h>
#include <ArduinoJson.h>
#include <BlynkSimpleEsp32.h>
#include <Adafruit_NeoPixel.h>
#include <time.h>
#include <Audio.h>

char ssid[] = "";
char pass[] = "";

BlynkTimer timer;
Adafruit_NeoPixel strip(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);
Audio audio;


boolean isSpeaking = false;

bool isAlarmListening = false;
bool alarmRinging = false;
bool isAlarmActive = false;
bool ledState = false;
unsigned long previousMillis = 0;
const long alarmInterval = 500;
String sliderTimeValue = "";
bool isWakingUp = false;
bool isClockOn = false;
bool spoke = false;
int countLoops = 0;

void setup()
{
  Serial.begin(115200);
  pinMode(SOUND_SENSOR_PIN, INPUT);
  pinMode(LIGHT_SENSOR_PIN, INPUT);
  pinMode(LED_PIN, OUTPUT);
  pinMode(BUZZER_PIN, OUTPUT);
  strip.begin();
  strip.setBrightness(255);
  strip.show();

  audio.setPinout(I2S_BCLK, I2S_LRC, I2S_DOUT);
  audio.setVolume(100);

  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, pass);
  configTime(2 * 3600, 3600, "pool.ntp.org");
  Blynk.virtualWrite(TIME_SLIDER, 21.48);
}

void loop()
{
  Blynk.run();
  if ((isAlarmListening && (readSound() || readLight())) || isAlarmActive ) {
    activateAlarm();
  }
  else {
    deactivateAlarm();
  }
  String currentTime = getCurrentTime();
  Blynk.syncVirtual(TIME_SLIDER);
  if (isClockOn) {
    if ( (currentTime == sliderTimeValue || isWakingUp)) {
      wakeup();
    }
  }
  else {
    deactivateClock();
  }
   
  if (!isWakingUp && !isClockOn && !spoke && countLoops > 1) {
    Serial.println("speak here");
    String text = getSpokenMessage();
    String first = text.substring(0, 96);
    speek(first.c_str());
    String second = text.substring(96);
    speek(second.c_str());
    spoke = true;
  }
}

void deactivateClock() {
  isWakingUp = false;
  Blynk.virtualWrite(CLOCK_BUTTON, 0);
  strip.clear();
  strip.show();
  noTone(BUZZER_PIN);
}

void wakeup() {
  countLoops++;
  isWakingUp = true;
  spoke = false;
  Serial.println("WakeUp!!!");
  Blynk.virtualWrite(CLOCK_BUTTON, 1);

  unsigned long currentMillis = millis();
  if (isClockOn) {
    for (int i = 0; i < strip.numPixels(); i++) {
      strip.setPixelColor(i, strip.Color(255, 255, 255)); // White color
    }
    strip.show();
    if (currentMillis - previousMillis >= alarmInterval) {
      previousMillis = currentMillis;
      static int loopsCount = 0;
      ledState = !ledState;

      if (ledState) {
        tone(BUZZER_PIN, 1500);
        delay(1000);
      } else {
        noTone(BUZZER_PIN);
      }
    }
  } else {
    strip.clear();
    strip.show();
    noTone(BUZZER_PIN);
  }
}



void activateAlarm() {
  isAlarmActive = true;
  Serial.println("Alarm Activated");
  Blynk.virtualWrite(ALARM_SYSTEM_ACTIVE_BUTTON, 1);

  unsigned long currentMillis = millis();
  if (isAlarmActive) {
    if (currentMillis - previousMillis >= alarmInterval) {
      previousMillis = currentMillis;
      ledState = !ledState;

      if (ledState) {
        // Set all pixels to red
        for (int i = 0; i < strip.numPixels(); i++) {
          strip.setPixelColor(i, strip.Color(255, 0, 0)); // Red color
        }
        strip.show();
        tone(BUZZER_PIN, 1000);
      } else {
        // Turn off all pixels
        strip.clear();
        strip.show();
        noTone(BUZZER_PIN);
      }
    }
  } else {
    strip.clear();
    strip.show();
    noTone(BUZZER_PIN);
  }
}

void deactivateAlarm() {
  isAlarmActive = false;
  Blynk.virtualWrite(ALARM_SYSTEM_ACTIVE_BUTTON, 0);
  strip.clear();
  strip.show();
  noTone(BUZZER_PIN);
}



bool readSound() {
  int sound = analogRead(SOUND_SENSOR_PIN);
  sound = sound > 1950 ? 1 : 0;
  bool ans = sound == 0 ? false : true;
  //    Serial.print("sound: ");
  //  Serial.println(sound);
  return ans;
}
bool readLight() {
  int light = analogRead(LIGHT_SENSOR_PIN);
  light = light > 3500 ? 0 : 1;
  bool ans = light == 0 ? false : true;
  //    Serial.print("light:");
  //  Serial.println(light);
  return light;
}


BLYNK_WRITE(CLOCK_BUTTON)
{
  int value = param.asInt();
  isWakingUp = false;
  isClockOn = false;
  Blynk.virtualWrite(TIME_SLIDER, -1);
  Blynk.virtualWrite(CLOCK_BUTTON, 0);
}

BLYNK_WRITE(ALARM_SYSTEM_LISTENING_BUTTON)
{
  // Set incoming value from pin V4 to a variable
  int value = param.asInt();
  isAlarmListening = value == 1 ? true : false;
}

BLYNK_WRITE(ALARM_SYSTEM_ACTIVE_BUTTON) {
  int value = param.asInt();
  if ( value == 0 && isAlarmListening) {
    isAlarmListening = false;
    isAlarmActive = false;
    Blynk.virtualWrite(ALARM_SYSTEM_LISTENING_BUTTON, 0);
  }
  if (value == 1 && !isAlarmListening) {
    Blynk.virtualWrite(ALARM_SYSTEM_ACTIVE_BUTTON, 0);
  }
}
BLYNK_WRITE(TIME_SLIDER) {
  double timeValue = param.asDouble();
  isClockOn = int(timeValue) != -1;
  int hours = int(timeValue);
  int minutes = int((timeValue - hours) * 100);
  String formattedTime = "";
  if (hours < 10) {
    formattedTime += "0";
  }
  formattedTime += String(hours);
  formattedTime += ":";
  if (minutes < 10) {
    formattedTime += "0";  // Add leading zero if minutes is less than 10
  }
  formattedTime += String(minutes);
  sliderTimeValue = formattedTime;
}


void myTimerEvent()
{
  //  Blynk.virtualWrite(V2, millis() / 1000);
}


String getCurrentTime() {
  // Get current time in seconds since the epoch
  time_t now;
  time(&now);
  struct tm *timeinfo;
  timeinfo = localtime(&now);
  // adjust to winter time
  int hours = timeinfo->tm_hour - 1;
  int minutes = timeinfo->tm_min;
  String timeAns = "";
  if (hours < 10) {
    timeAns = timeAns + "0";
  }
  timeAns = timeAns + String(hours) + String(":");
  if (minutes < 10) {
    timeAns = timeAns + String("0");
  }
  timeAns = timeAns + String(minutes);
  //  Serial.print("Current time: ");
  //  Serial.println(timeAns);
  return timeAns;
}



String getSpokenMessage() {
  HTTPClient http;

  const char* url = "http://dailyinfo.hheghmbdgxezfjew.israelcentral.azurecontainer.io/daily_data";

  http.begin(url);
  int httpResponseCode = http.GET();

  String spokenMessage = "";

  if (httpResponseCode > 0) {
    DynamicJsonDocument jsonBuffer(1024 * 4);
    DeserializationError error = deserializeJson(jsonBuffer, http.getStream());

    if (!error) {
      // Extract spoken_message field
      spokenMessage = jsonBuffer["spoken_message"].as<String>();
    } else {
      Serial.print("JSON parsing failed: ");
      Serial.println(error.c_str());
    }
  } else {
    Serial.print("HTTP GET request failed, error: ");
    Serial.println(httpResponseCode);
  }

  // Close connection
  http.end();

  return spokenMessage;
}


void speek(const char* text) {
  Serial.print("saying text: "); Serial.println(text);
  audio.connecttospeech(text, "en"); // Google TTS
  while (isSpeaking == false) {
    audio.loop();
  }
  Serial.println("finished speaking");
  isSpeaking = false;
}

void audio_info(const char *info) {
  if (String(info).startsWith("End of speech")) {
    isSpeaking = true;
  }
  Serial.print("audio_info: "); Serial.println(info);
}
